import pandas as pd
import numpy as np
from pandas import DataFrame
from typing import Optional, Iterable, Tuple, List, Union


def get_sep(file_path: str) -> str:
    """Figure out the sep based on file name. Only helps with tsv and csv.

    Args:
        file_path: Path of file.

    Returns: sep

    """
    if file_path[-4:] == '.tsv':
        return '\t'
    elif file_path[-4:] == '.csv':
        return ','
    raise ValueError('Input file is not a .csv or .tsv')


def read_protein(file_path: str) -> DataFrame:
    """Reads in protein abundance values. Proteins as rows, samples as columns.
    First column must be protein identifier.

    Args:
        file_path: Path to protein csv or tsv.

    Returns: DataFrame with proteins as rows, samples as columns.

    """
    sep = get_sep(file_path)
    return pd.read_csv(file_path, sep=sep, index_col=0).replace(
        ['na', 'NA', 'NAN', 'nan', 'NaN', 'Na'], np.nan
    ).astype(float)

def read_gct(path: str, 
             index_cols: Optional[List[str]]=['geneSymbol', 'variableSites'], 
             regex: Optional[Union[str, None]]=None, 
             sample_cols: Optional[Union[List, None]]=None, 
             annotation_rows: Optional[Union[List, None]]=None
            ) -> Tuple[DataFrame, DataFrame]:
    """
    Reads in a gct file and formats the dataframe so it's ready for phospho disco
    path: path to file.gct
    index_cols:  columns which to use as an index. For phospho/acetyl, etc this should 
                 be two columns e.g. ['geneSymbol', 'variableSites'] whereas for protein it's one column, e.g. ['geneSymbol']
    regex:       [optional] regular expression to quickly subselect sample columns e.g.  
                 to get only sample columns that end in a digit: r'.*\d$'
    sample_cols: [optional] to select sample columns using exact names; not used if you provided a regex
    
    returns: pd.DataFrame with sample columns and index_cols
    """
    with open(path, 'r') as handle:
        next(handle)
        #the 2nd row of a gct file gives us the dimensions
        nrows, ncols, nrowmeta, ncolsmeta = [int(i) for i in next(handle).split()] 
    df = pd.read_csv(
        path, sep='\t', skiprows=2, low_memory=False
    ).replace(
         ['na', 'NA', 'NAN', 'nan', 'NaN', 'Na'], np.nan
    )
    # the metadatatable is transposed in the gct file, hence we are indexing everything but 
    # the first ncolsmeta rows, and everything but the first nrowsmeta columns
    sample_df = df.set_index(index_cols).iloc[ncolsmeta:, nrowmeta-len(index_cols)+1:].copy()
    annots_df = df.set_index(df.columns[0]).iloc[:ncolsmeta, nrowmeta:].copy()
    if regex is not None:
        sample_df = sample_df.loc[:,sample_df.columns.str.match(regex)]
        annots_df = annots_df.loc[:,annots_df.columns.str.match(regex)]
    elif sample_cols is not None:
        try:
            sample_df = sample_df.loc[:, sample_cols]
            annots_df = annots_df.loc[:, sample_cols]
        except KeyError:
            non_matched_cols = set(sample_cols).difference(sample_df.columns)
            raise IndexError(
                f"The following columns were not found in the sample columns of the provided gct file \npath:\n{path}\
                \nmismatched columns:\n{non_matched_cols}"
            )
    if annotation_rows is not None:
        try:
            annots_df = annots_df.loc[annotation_rows, :]
        except KeyError:
            non_matched_cols = set(annotation_rows).difference(annots_df.index)
            raise IndexError(
                f"The following columns were not found in the annotation rows columns of the provided gct file \npath:\
                \n{path}\nmismatched columns:\n{non_matched_cols}"
            )

    return sample_df, annots_df

def filter_dups(group:pd.DataFrame):
    """
    Meant to be used in apply after a groupby call.
    For a set of rows (group) find the one with the lowest number of NaNs or tied for the lowest number.
    group: pd.DataFrame
    """
    nan_counts = group.apply(lambda r: r.isnull().sum(), axis=1)
    min_nan_counts = nan_counts.min()
    return group.loc[nan_counts == min_nan_counts].head(1)

# def deduplicate_rows(
#     df: pd.DataFrame,
#     samples,
#     sequence_col = 'sequence',
#     maximize_cols = ['bestScore', 'Best_scoreVML', 'bestDeltaForwardReverseScore']
#     ) -> pd.DataFrame:

#     groupby = df.index.names
#     df = df.reset_index()

#     df[maximize_cols] = df[maximize_cols].astype(float)
#     df['numNAs'] = df[samples].isnull().sum(axis=1)
#     if sequence_col:
#         df['len'] = df[sequence_col].str.len()
#         return df.groupby(groupby).apply(lambda row: row.nsmallest(1, columns=['numNAs','len'], keep='all').nlargest(1, columns=maximize_cols, keep='first')).reset_index(level=-1, drop=True)
#     return df.groupby(groupby).apply(lambda row: row.nsmallest(1, columns=['numNAs'], keep='all').nlargest(1, columns=maximize_cols, keep='first')).reset_index(level=-1, drop=True)


def read_annotation(file_path: str) -> DataFrame:
    """Reads in sample annotation file. Sample as rows, annotations as columns.

    Args:
        file_path: Path to protein csv or tsv. First column must be sample identifier.

    Returns: DataFrame with samples as rows, annotations as columns.

    """
    sep = get_sep(file_path)
    return pd.read_csv(file_path, sep=sep, index_col=0).replace(
        ['na', 'NA', 'NAN', 'nan', 'NaN', 'Na'], np.nan
    )


def read_phospho(file_path: str) -> Optional[DataFrame]:
    """Reads in protein abundance values. Proteins as rows, samples as columns. First two columns
    must be protein, variable stie identifiers, respectively. Can use this for raw or normalized
    phospho data tables.

    Args:
        file_path: Path to protein csv or tsv.

    Returns: DataFrame with phosphosites as rows, samples as columns.

    """
    sep = get_sep(file_path)
    return pd.read_csv(file_path, sep=sep, index_col=[0, 1]).replace(
        ['na', 'NA', 'NAN', 'nan', 'NaN', 'Na'], np.nan
    ).astype(float)


def read_list(file_path: str):
    """Reads in a \n separated file of things into a list.

    Args:
        file_path: Path to file.

    Returns: List

    """
    with open(file_path, 'r') as fh:
        return [s.strip() for s in fh.readlines()]
    

def column_normalize(df: DataFrame, method: str) -> DataFrame:
    """Normalizes samples for coverage.

    Args:
        df: DataFrame to column normalize.
        method: Which method to use: 'median_of_ratios', 'median', 'upper_quartile' currently
        accepted.

    Returns: Normalized DataFrame.

    """
    if method == "median_of_ratios":
        return df.divide(df.divide(df.mean(axis=1), axis=0).median())

    if method == "median":
        return df.divide(df.median())

    if method == "upper_quartile":
        return df.divide(np.nanquantile(df, 0.75))

    # if method == "quantile":
    #     pass
        #TODO add two comp

    raise ValueError(
        'Passed method not valid. Must be one of: median_of_ratios, median, upper_quartile, '
        'twocomp_median.'
    )


def read_fasta(fasta_file) -> dict:
    """Parse fasta into a dictionary.

    Args:
        fasta_file: path to fasta file.

    Returns: dictionary of genes: seq.

    """
    with open(fasta_file, 'r') as fh:
        aa_seqs = {
            seq.split()[0]: seq.split(']')[-1].replace('\s', '').replace('\n', '')
            for seq in fh.read().strip().split('>') if seq != ''
        }
    return aa_seqs


def read_gmt(gmt_file: str) -> dict:
    """Parser for gmt files, specifically from ptm-ssGSEA

    Args:
        gmt_file: Name of gmt file.

    Returns:
        Dictionary of ptm sets. Keys are labels for each set. Values are dictionaries with
        structure: {aa sequence: site name}

    """
    result = {}
    with open(gmt_file, 'r') as fh:
        for line in fh.readlines():
            line = line.strip().split('\t')
            name = line[0]
            site_labels = line[1]
            seqs = line[2:]
            seq_labels = {seqs[i].split('-')[0]: label for i, label in enumerate(site_labels.split('|')[1:])}
            result.update({name: seq_labels})

    return result



module_combiner_delim = '-'
annotation_column_map = {'categorical': 0, 'continuous': 1, '1':1, '0':0}
datatype_label = 'datatype_label'
var_site_delimiter = ','
protein_id_col = 'protein_id'
variable_site_col = 'variable_sites'
variable_site_aa_col = 'variable_sites_names'
seq_col = 'aas'
gene_symbol_col = 'gene_symbol'
module_name_col = 'module'




import pandas as pd
import numpy as np
from pandas import DataFrame
from typing import Iterable, Optional
from sklearn import linear_model, preprocessing
from .utils import corr_na, zscore


def collapse_possible_regulators(
        reg_data: DataFrame,
        corr_threshold: float = 0.95
) -> DataFrame:
    """Uses mean to collapse rows of possible regulator data that are highly correlated. Slightly
    chaotic, since it just averages two at a time based on iterating through a dictionary. Use
    with caution.

    Args:
        reg_data: DataFrame with possible regulator features as rows, samples as columns.
        corr_threshold: Rows with pearson correlation higher than this value will be averaged
        iteratively until there are no more rows with more than this correlation.

    Returns: DataFrame with rows that do not have pairwise correlation above the corr_threshold.

    """
    reg_data = zscore(reg_data)
    corr = reg_data.transpose().corr()
    
    high_corr_inds = corr.index[((corr > corr_threshold).sum(axis=1) > 1)]
    low_corr_inds = corr.index.difference(high_corr_inds)
    high_corr_data = reg_data.loc[high_corr_inds, :]
    low_corr_data = reg_data.loc[low_corr_inds, :]
    if len(high_corr_inds) == 0:
        return low_corr_data
    corr = corr.mask(
        np.tril(np.ones(corr.shape)).astype(np.bool)
    ).mask(~(corr > corr_threshold)).dropna(how='all').dropna(how='all', axis=1)
    corr = corr.stack(level=[0, 1])

    while corr.shape[0] > 0:
        for i in corr.index:
            a, b, c, d = i
            if (a, b) in high_corr_data.index and (c, d) in high_corr_data.index:
                inds_to_mean = [(a, b), (c, d)]
                
                others_ab = [
                    (d, e, f, g) for d, e, f, g in corr.index 
                    if (d, e) is (a, b) or (f, g) is (a, b) 
                ]
                others_ab = [(f, g) if (d, e) == (a, b) else (d, e) for d, e, f, g in others_ab]
                inds_to_mean.extend(
                    [(e, f) for e, f in others_ab
                     if ((e, f, c, d) in high_corr_data.index)
                     or ((c, d, e, f) in high_corr_data.index)]
                )
            
                name = ('%s-%s' %(a, c), '%s-%s' % (b, d))
                high_corr_data = high_corr_data.append(
                    pd.Series(high_corr_data.loc[inds_to_mean, :].mean(), name=name)
                )
                high_corr_data = high_corr_data.drop(inds_to_mean, axis=0)
        corr = high_corr_data.transpose().corr()
        high_corr_inds = corr.index[((corr > corr_threshold).sum(axis=1) > 1)]
        low_corr_inds = corr.index[((corr > corr_threshold).sum(axis=1) <= 1)]

        low_corr_data = low_corr_data.append(high_corr_data.loc[low_corr_inds, :])
        if len(high_corr_inds) == 0:
            return low_corr_data
        high_corr_data = high_corr_data.loc[high_corr_inds, :]
        corr = corr.mask(
            np.tril(np.ones(corr.shape)).astype(np.bool)
        ).mask(~(corr > corr_threshold)).dropna(how='all').dropna(how='all', axis=1)
        corr = corr.stack(level=[0, 1])

    return low_corr_data


def calculate_regulator_coefficients(
        reg_data: DataFrame,
        module_scores: DataFrame,
        scale_data: bool = True,
        model: str = 'linear',
        regularization_values: Optional[Iterable] = None,
        cv_fold: int = 5,
        **model_kwargs
) -> DataFrame:
    """Calculates linear model coefficients between regulator data and module scores.

    Args:
        reg_data: DataFrame with possible regulator features as rows, samples as columns.
        module_scores: DataFrame with module scores as rows, samples as columns.
        scale_data: Whether to scale the regulator data and module scores with
        sklearn.preprocess.scale before training the linear model.
        model: Whether the relationship between log2(regulator_data) and module scores should be
        modeled as linear or sigmoid.
        regularization_values: Which regularization values should be tried during CV to define
        the coefficients.
        cv_fold: The number of cross validation folds to try for calculating the regularization
        value.
        **model_kwargs: Additional keyword args for sklearn.linear_model.RidgeCV

    Returns: The first object is a DataFrame with module scores as rows and regulators as columns,
    and model coefficients as values. The second is a Series with module scores as rows and the
    over all model quality score as values.

    """
    if regularization_values is None:
        regularization_values = [5 ** i for i in range(-5, 5)]

    if model not in ['linear', 'sigmoid']:
        raise ValueError(
            'Model %s not in accepted models: %s' % (model, ','.join(['linear', 'sigmoid']))
        )

    features = reg_data.transpose().values
    targets = module_scores.transpose().values
    if model == 'sigmoid':
        targets = -np.log2(1+(2**-targets))
    if scale_data:
        features = preprocessing.scale(features, copy=True)
        targets = preprocessing.scale(targets, copy=True)
        
    model_kwargs.update({'cv': cv_fold, 'alphas':regularization_values})
    model = linear_model.RidgeCV(**model_kwargs)
    model.fit(features, targets)
    weights = pd.DataFrame(
        model.coef_,
        index=module_scores.index,
        columns=reg_data.index
    ).transpose()
    scores = pd.Series(
        model.score(features, targets),
        index=module_scores.index,
    )
    return weights, scores


def calculate_regulator_corr(
        reg_data: DataFrame,
        module_scores: DataFrame,
        **model_kwargs
):
    """Calculates the correlation between possible regulators and module scores.

    Args:
        reg_data: DataFrame with possible regulator features as rows, samples as columns.
        module_scores: DataFrame with module scores as rows, samples as columns.
        **model_kwargs: Additional keyword args for corr_na, including method which can take
        either pearsonr or spearmanr.

    Returns: Two DataFrames, both with module scores as rows and possible regulators as columns.
    The first one has correlation R values, the second has correlation p values.

    """
    rs = pd.DataFrame(index=reg_data.index)
    ps = pd.DataFrame(index=reg_data.index)
    for i, row in module_scores.iterrows():
        res = reg_data.apply(lambda r: pd.Series(corr_na(row, r, **model_kwargs)), axis=1)
        rs[i] = res.iloc[:, 0]
        ps[i] = res.iloc[:, 1]
    return rs, ps



from typing import Optional, List, Iterable
import sys
import logging
import argparse
import yaml
import oyaml
import numpy as np
import phosphodisco
import pathlib
import importlib
from io import BytesIO
import pkgutil
import subprocess

logging.basicConfig(
    format='%(asctime)s %(levelname)-8s %(message)s',
    level=logging.INFO,
    datefmt='%Y-%m-%d %H:%M:%S')

logger = logging.getLogger('PhosphoDisco')


def _make_parser(fun=None, help_text=None):
    """
    Makes a parser for the clis.
    """
    parser = argparse.ArgumentParser(prog="phosphodisco", description=help_text)
    parser.add_argument(
        "--version", "-v", action="version", version="%s" % phosphodisco.__version__
    )
    if fun == 'generate_config':
        parser.add_argument(
                "--config_path", default='phdc_custom_config.yml', type=pathlib.Path, 
                help='path where config should be output'
        )        
        parser.add_argument(
                "--template", default=0, choices=[0, 1, 2], type=int,
                help='which config template to use - 0 was used for phosphodisco publication, 1 contains additional clustering methods (LouvainCluster and LeidenCluster), 2 is the template used for the demo/tutorial'
        )        
        parser.add_argument(
                "--phospho", type=pathlib.Path, required=True, 
                help='path to phospho file'
        )
        parser.add_argument(
                "--protein", type=pathlib.Path, required=True, 
                help='path to protein file'
        )
        parser.add_argument(
                "--sample_table", type=pathlib.Path, default=None,
                help='path to a sample annotation table csv file. First column has sample ids.'
        )
        parser.add_argument(
                "--sample_columns", nargs='+', default=None,
                help='which columns from sample annotation table to use. Will use all cols if none provided.'
        )
        parser.add_argument(
                "--min_common_values", type=int, default=6, help=''
        )
        parser.add_argument(
                 "--top_stdev_quantile", type=float, default=0.5, help=''
        )
        parser.add_argument(
                "--na_frac_threshold", type=float, default=0.25, help=''
        )

    elif fun == 'run':
        parser.add_argument(
                "--config_file", type=pathlib.Path, required=True,
                help='path to snakemake config file to be used by pipeline. Use phdc_generate_config to geneerate one from a template'
        )
        parser.add_argument(
                "--cores", type=int, default=3, 
                help='how many cores to use. default=3'
        )
        parser.add_argument(
                "--jobs", type=int, default=20, 
                help='how many jobs to run at a time. default=20'
        )
        parser.add_argument(
                "--cluster_config", type=pathlib.Path, 
                help='snakemake cluster config file'
        )
        parser.add_argument(
                "--dry_run", action='store_const', const='-n', default='',
                help='perform a dry-run of the pipeline, i.e. just print what would be run'
        )
        parser.add_argument(
                "--cluster_submit_command", default='',
                help='cluster submit command to feed to snakemake. Will run in local mode if not provided.\nslurm example:\n"sbatch --mem={cluster.mem} -t {cluster.time} -o {cluster.output} --cpus-per-task {cluster.cpus-per-task}"'
        )
        

# snakemake --snakefile phdc.smk --cores 3 -n --configfile config-custom.yml --cluster-config cluster.json

    return parser

def run():
    """
    Runs the snakemake phosphodisco workflow from the command line.
    """
    help_text="""Runs the snakemake phosphodisco workflow from the command line."""
    logger.info('run()\n'+ help_text)
    parser = _make_parser(fun='run', help_text=help_text)
    args = parser.parse_args()
    phdc_path = importlib.util.find_spec('phosphodisco')
    phdc_path = pathlib.Path(phdc_path.origin).parent
    snakefile = phdc_path / 'phdc.smk'
    snakemake_cmd = [
        'snakemake', 
        '--snakefile', snakefile, 
        '--cores', str(args.cores), 
        '--configfile', args.config_file,
        args.dry_run,
        f'--cluster-config {args.cluster_config}' if args.cluster_config else '',     
        f"--cluster '{args.cluster_submit_command}'" if args.cluster_submit_command else '',
        f"-j {args.jobs}" if args.jobs else '',

    ]
    snakemake_cmd = list(filter(lambda it: it!='', snakemake_cmd))
    logger.info(' '.join(['command run:\n'] + [str(i) for i in snakemake_cmd]))
    subprocess.run(
        ' '.join([str(i) for i in snakemake_cmd]),
        shell=True
    )

def generate_config():
    """
    Makes a copy of the config template and modifies it according to the flags.
    """
    help_text="""Generates a config file to be used by phdc_run."""
    parser = _make_parser(fun='generate_config', help_text=help_text)
    args = parser.parse_args()
    config_template_dict = {
            0:'data/config-custom_template0.yml', 
            1:'data/config-custom_template1.yml',
            2:'data/config-custom_template2.yml'
            }
    config_template = BytesIO(pkgutil.get_data('phosphodisco', config_template_dict[args.template])) 
    template_yml = oyaml.load(config_template, Loader=oyaml.FullLoader)
    if args.template == 2:
        # template 2 is used for the demo
        # only the paths have to be set, everything else is set in the template already
        template_yml['input_phospho'] = pathlib.Path(__file__).parent / 'data' / 'demo' / 'BRCA_v5.2__selected_phospho.csv'
        template_yml['input_protein'] = pathlib.Path(__file__).parent / 'data' / 'demo' / 'BRCA_v5.2__selected_protein.csv'
    else:
        template_yml['input_phospho'] = str(args.phospho)
        template_yml['input_protein'] = str(args.protein)
        template_yml['std_quantile_threshold'] = args.top_stdev_quantile
        template_yml['min_common_vals'] = args.min_common_values
        template_yml['na_frac_threshold'] = args.na_frac_threshold
       #add sample table and sample columns info 
        template_yml['sample_annotations_csv'] = str(args.sample_table) if args.sample_table is not None else None
        template_yml['sample_annot_cols_for_normalization'] = args.sample_columns

    # warnings in case user mistypes phospho/protein paths:
    for field, path in {'phospho':args.phospho, 'protein':args.protein}.items():
        if not pathlib.Path(path).exists():
            logger.warning(f'The following {field} path does not exist: {path}')
    # write new custom config to file
    with open(args.config_path, 'w') as fh:
        fh.write(oyaml.dump(template_yml))


def _main(args: Optional[List[str]] = None):
    if args is None:
        args = sys.argv[1:]
    args = _make_parser().parse_args(args)
    logger.info("Running phosphodisco")

    for arg in vars(args):
        logger.info("Parameter %s: %s" % (arg, getattr(args, arg)))

    output_prefix = args.output_prefix
    phospho = phosphodisco.parsers.read_phospho(args.phospho)
    protein = phosphodisco.parsers.read_protein(args.protein)
    min_common_values = args.min_common_values

    if args.normed_phospho:
        normed_phospho = phosphodisco.parsers.read_phospho(args.normed_phospho)
    else:
        normed_phospho = None

    if args.modules:
        modules = phosphodisco.parsers.read_phospho(args.modules)
    else:
        modules = None

    if args.additional_kwargs_yml:
        with open(args.additional_kwargs_yml, 'r') as fh:
            additional_kwargs_yml = yaml.load(fh, Loader=yaml.FullLoader)
    else:
        additional_kwargs_yml = {}

    data = phosphodisco.ProteomicsData(
        phospho=phospho,
        protein=protein,
        min_common_values=min_common_values,
        normed_phospho=normed_phospho,
        modules=modules,
    )
    logger.info("Instantiated ProteomicsData")
    if args.normed_phospho is None:
        logger.info('Normalizing phospho by protein')
        data.normalize_phospho_by_protein(
            **additional_kwargs_yml.get('normalize_phospho_by_protein', {})
        )
        logger.info("Finished normalizing phospho by protein")
        #TODO make it so this doesn't keep writing this file
        data.normed_phospho.to_csv('%s.normed_phospho.csv' % output_prefix)
    if data.normed_phospho.isnull().any().any():
        logger.info("Imputing missing values")
        data.impute_missing_values(**additional_kwargs_yml.get('impute_missing_values', {}))
        logger.info("Finished imputing missing values")
        data.normed_phospho.to_csv('%s.normed_phospho.csv' % output_prefix)
    if args.top_stdev_percent < 100:
        data.normed_phospho = data.normed_phospho.loc[
            data.normed_phospho.std(axis=1)<np.percentile(
                data.normed_phospho.std(axis=1), 100-args.top_stdev_percent
            )
        ]
        data.normed_phospho.to_csv('%s.normed_phospho.top%sstdev_percent.csv' % (output_prefix, args.top_stdev_percent))
    
    
    if args.stop_before_modules:
        logger.info('Stopping before calculating modules, PhosphoDisco done')
        return None
    
    if args.modules is None:
        logger.info("Assigning modules")
        data.assign_modules(
            force_choice=True, **additional_kwargs_yml.get('assign_modules', {})
        )
        data.modules.to_csv('%s.modules.csv' % output_prefix)
        logger.info("Finished assigning modules")
    data.calculate_module_scores(
        **additional_kwargs_yml.get('calculate_module_scores', {})
    )
    logger.info("Calculated module scores")

    if args.putative_regulator_list:
        with open(args.putative_regulator_list, 'r') as fh:
            putative_regulator_list = [gene.strip() for gene in fh.readlines()]
        data.collect_possible_regulators(
            putative_regulator_list, **additional_kwargs_yml.get('collect_possible_regulators', {})
        )
        logger.info("Collected possible regulators")
        data.calculate_regulator_association(
            **additional_kwargs_yml.get('calculate_regulator_coefficients', {})
        )
        data.regulator_coefficients.to_csv('%s.putative_regulator_coefficients.csv' % output_prefix)
        logger.info("Calculated regulator coefficients")

    if args.annotations:
        if args.annotation_column_types is None:
            logger.error(
                'Annotations were provided, but no column labels were provided. Cannot continue '
                'with annotation association calculations. '
            )
            return None
        annotations = phosphodisco.parsers.read_annotation(args.annotations)
        with open(args.annotation_column_types, 'r') as fh:
            annotation_column_types = [col.strip() for col in fh.readlines()]
        data.add_annotations(
            annotations=annotations,
            column_types=annotation_column_types
        )
        logger.info("Added annotations")
        data.calculate_annotation_association(
            **additional_kwargs_yml.get('calculate_annotation_association', {})
        )
        data.annotation_association.to_csv('%s.annotation_association.csv' % output_prefix)
        logger.info("Calculated annotation association. ")
        
    logger.info("PhosphoDisco done")


if __name__ == "__main__":
    _main()



import pandas as pd, numpy as np, seaborn as sns, phosphodisco as phdc

modules=pd.read_csv(
'/gpfs/data/ruggleslab/phosphodisco/datasets/harmonized_sets/clustering/lssc_vs_luad/matched_tumors_and_normals_prot-normed_indiv_by_nat_tum_and_lscc-v3.2_vs_luad-v3.0-phospho_0.75frac-no-na.top50frac_stdev_filt_3_levels_0.75frac-no-na.top50frac_stdev.corr/clustering_intermediates/HDBSCAN;min_cluster_size-9_labels.txt',
index_col=[0,1]
    )
modules = modules.loc[modules.iloc[:,0] != -1] #drop phosphosites that did not fit into any module.
modules.head(2)

normed = pd.read_csv(
'/gpfs/data/ruggleslab/phosphodisco/datasets/harmonized_sets/lssc_vs_luad/matched_tumors_and_normals_prot-normed_indiv_by_nat_tum_and_lscc-v3.2_vs_luad-v3.0-phospho_0.75frac-no-na.top50frac_stdev_filt_3_levels.normed.csv',
index_col=[0,1]
)
normed.head(2)

phospho = pd.read_csv('/gpfs/data/ruggleslab/phosphodisco/datasets/harmonized_sets/lssc_vs_luad/matched_normals_lscc-v3.2_vs_luad-v3.0-phospho.csv', index_col=[0,1])
 
prot = pd.read_csv('/gpfs/data/ruggleslab/phosphodisco/datasets/harmonized_sets/lssc_vs_luad/matched_tumors_and_normal_lscc-v3.2_vs_luad-v3.0-prot.clean.csv', index_col=[0]
).loc[:, normed.columns]
prot.head(2)

proteomics_obj = phdc.ProteomicsData(
 phospho=phospho,
 protein=prot,
 normed_phospho=normed,
 modules=modules,
)

with open('/gpfs/data/ruggleslab/phosphodisco/datasets/utilities/GRCh38_latest_protein.faa','r') as fh:
    aa_seqs = {seq.split()[0]: seq.split(']')[-1].replace('\s', '').replace('\n', '') for seq in fh.read().strip().split('>') if seq != ''}

lscc_raw_phospho = pd.read_csv('/gpfs/data/ruggleslab//pancan/LSCC/lscc-v3.2-data-freeze/lscc-v3.2-phosphoproteome-ratio-norm-NArm.gct', skiprows=2, sep='\t')\
.replace('na', np.nan)\
.dropna(subset=['geneSymbol'])\
.set_index(['geneSymbol', 'variableSites'])
lscc_peptide_ids = lscc_raw_phospho.loc[normed.index.intersection(lscc_raw_phospho.index), 'id'].str.split('.')
lscc_peptide_ids = lscc_peptide_ids.str.get(0) + '.' + lscc_peptide_ids.str.get(1).str.split('_').str.get(0)
lscc_peptide_ids.head(2)

luad_raw_phospho = pd.read_csv('/gpfs/data/ruggleslab/phosphodisco/datasets/harmonized_sets/luad/luad-v3.0-phospho-isoform-id-dedup.csv').set_index(['geneSymbol', 'variableSites'])
luad_peptide_ids = luad_raw_phospho.loc[normed.index.intersection(luad_raw_phospho.index), 'accession_number'].str.split('.')
luad_peptide_ids = luad_peptide_ids.str.get(0) + '.' + luad_peptide_ids.str.get(1).str.split('_').str.get(0)
luad_peptide_ids.head(2)

combined_peptide_ids = pd.concat([luad_peptide_ids, lscc_peptide_ids]).reset_index().rename(columns={0:'accession_number'})\
.drop_duplicates().set_index(['geneSymbol', 'variableSites'])
combined_dup_peptide_ids = combined_peptide_ids.loc[combined_peptide_ids.index.duplicated(keep=False)].sort_index()
combined_dedup_peptide_ids = combined_peptide_ids.loc[combined_peptide_ids.index.duplicated(keep='first')+(~combined_peptide_ids.index.duplicated(keep=False)) ]

raw_modules=pd.read_csv(
'/gpfs/data/ruggleslab/phosphodisco/datasets/harmonized_sets/clustering/lssc_vs_luad/matched_tumors_and_normals_prot-normed_indiv_by_nat_tum_and_lscc-v3.2_vs_luad-v3.0-phospho_0.75frac-no-na.top50frac_stdev_filt_3_levels_0.75frac-no-na.top50frac_stdev.corr/clustering_intermediates/HDBSCAN;min_cluster_size-9_labels.txt',
index_col=[0,1]
)
raw_modules.head(2)

import re
seq_df = combined_dedup_peptide_ids.loc[raw_modules.index].copy()
seq_df[raw_modules.iloc[:,0].name] = raw_modules.iloc[:,0]
seq_df = seq_df.reset_index()
seq_df['variable_sites'] = seq_df['variableSites'].apply(lambda x: re.sub("[^0-9\s]", "", x).strip().replace(' ', ','))
seq_df = seq_df.rename(columns = {'accession_number':'protein_id', 'variableSites':'variable_sites_names', 'geneSymbol':'gene_symbol'})
seq_df

proteomics_obj.collect_aa_sequences(seq_df, aa_seqs, 'HDBSCAN;min_cluster_size-9', n_flanking=7, var_sites_aa_col='variable_sites_names')
print('Calculating aa_overlap')
proteomics_obj.analyze_aa_overlap()
#module_overlap_df_dict = phdc.motif_analysis.aa_overlap_from_df(proteomics_obj.module_seq_df, module_col='HDBSCAN;min_cluster_size-9')
print('Plotting now')
phdc.visualize.visualize_aa_overlap(module_overlap_df_dict=proteomics_obj.module_overlap_df_dict, save_path='lmao')



from .classes import ProteomicsData
from pathlib import Path
from pandas import DataFrame
import pandas as pd
import numpy as np
from typing import Optional
import matplotlib
import matplotlib.pyplot as plt
import seaborn as sns
from .catheat import heatmap as catheat
import logomaker
from scipy.cluster import hierarchy
from scipy.spatial.distance import pdist


matplotlib.rcParams["pdf.fonttype"] = 42
matplotlib.rcParams["ps.fonttype"] = 42
sns.set(font='arial', style='white', color_codes=True, font_scale=1.3)
matplotlib.rcParams.update({'savefig.bbox': 'tight'})


def compute_order(
        df: DataFrame,
        optimal: bool = True,
        dist_method: str="euclidean",
        cluster_method: str="average"
):
    """Computes order of samples for clustered heatmaps.

    Args:
        df: Data with rows to cluster.
        optimal: Whether to return optimal ordering. Slows stuff down.
        dist_method: Which distance calculation to use.
        cluster_method: Which hierarchical clustering method to use.

    Returns: Clustered order of rows.

    """
    dist_mat = pdist(df, metric=dist_method)
    link_mat = hierarchy.linkage(dist_mat, method=cluster_method)

    if optimal==True:
        return hierarchy.leaves_list(hierarchy.optimal_leaf_ordering(link_mat, dist_mat))
    else:
        return hierarchy.leaves_list(link_mat)


def visualize_modules(
        data: ProteomicsData,
        annotations: Optional[DataFrame] = None,
        col_cluster=True,
        row_cluster=True,
        cluster_kws: dict = {},
        annot_kws: dict = {},
        heatmap_kws: dict = {},
        file_prefix: str = 'heatmap'
):
    """Makes heatmap figures of sites vs samples for each module.

    Args:
        data: ProteomicsData object containing AT LEAST normed_phospho data with no missing
        values (maybe imputed), and modules assigned.
        annotations: A DataFrame with samples as rows and categorical annotations to visualize as
        columns. This isn't taken directly from ProteomicsData because that table may have many
        more columns than anyone wants to visualize.
        col_cluster: Whether to cluster samples in the heatmaps.
        row_cluster: Whether to cluster rows in the heatmaps.
        cluster_kws: Additional keyword args to pass to visualize.compute_order
        annot_kws: Additional keyword args to pass to catheat.heatmap
        heatmap_kws: Additional keyword args to pass to sns.heatmap
        file_prefix: File prefix for each figure. Suffix will be .clusterX.pdf

    Returns: None

    """

    cluster_sets = data.modules
    cluster_sets = {
        cluster_name: cluster_sets.index[cluster_sets==cluster_name] for cluster_name in
        data.modules.unique() if cluster_name != -1
    }
    values = data.normed_phospho

    for cluster_name, sites in cluster_sets.items():
        if int(cluster_name) == -1:
            continue

        df = values.loc[sites, :]

        if row_cluster:
            row_order = compute_order(df, **cluster_kws)
            row_order = [df.index[i] for i in row_order]
        else:
            row_order = df.index

        if col_cluster:
            col_order = compute_order(df.transpose(), **cluster_kws)
            col_order = [df.columns[i] for i in col_order]
        else:
            col_order = df.columns
        df = df.reindex(col_order, axis=1).reindex(row_order, axis=0)
        if annotations is None:
            header = pd.DataFrame(np.empty(len(col_order)), columns=[''])
        else:
            header = annotations.reindex(col_order).transpose()

        fig_len = 0.25*(len(df) + len(header))
        fig_width = 0.15*len(col_order)

        _ = plt.figure(figsize=(fig_width, fig_len))
        gs = plt.GridSpec(
            nrows=3, ncols=2,
            height_ratios=[len(header)]+2*[len(df)/2],
            width_ratios=[len(col_order), 5],
            hspace=0, wspace=0
        )

        heat_ax = plt.subplot(gs[1:, 0])
        cbar_ax = plt.subplot(gs[-1, -1])
        if annotations is not None:
            header_ax = plt.subplot(gs[0, 0])
            leg_ax = plt.subplot(gs[1, 1])
            leg_ax.axis('off')
            catheat(
                header,
                xticklabels=False, yticklabels=header.index,
                ax=header_ax, leg_ax=leg_ax, leg_kws=dict(loc=(0,0), labelspacing=0),
                **annot_kws
            )
            header_ax.set_yticklabels(header.index, rotation=0)
            header_ax.set_title('Cluster %s' % cluster_name)
            header_ax.set_xlabel('')
            header_ax.set_ylabel('')

        sns.heatmap(
            df,
            xticklabels=df.columns, yticklabels=['-'.join(i) for i in df.index],
            ax=heat_ax,
            cbar_ax=cbar_ax,
            **heatmap_kws
        )
        heat_ax.set_yticklabels(heat_ax.get_yticklabels(), rotation=0)
        heat_ax.set_xlabel('')
        heat_ax.set_ylabel('')

        plt.savefig('%s.cluster%s.pdf' % (file_prefix, cluster_name))
        plt.show()
        plt.close()


def visualize_regulator_coefficients(
        data: ProteomicsData,
        percentile_cutoff: float = 95,
        savefig_prefix: Optional[str] = None,
        **heatmap_kwargs
):
    """Visualizes the associations between putative regulators and modules.

    Args:
        data: ProteomicsData object with regulator_coefficients assigned.
        percentile_cutoff: Heatmaps are filtered to show high associations only. What threshold
        should be used.
        savefig_prefix: If prefix is provided, a figure will be saved with this prefix.
        **heatmap_kwargs: Additional keyword args for sns.heatmap

    Returns: matplotlib ax with heatmap of coefficients

    """
    if data.regulator_coefficients is None:
        raise KeyError(
            'Must calculate regulator coefficients using '
            'ProteomicsData.calculate_regulator_coefficients before visualizing. '
        )
    cut_off = np.nanpercentile(
        data.regulator_coefficients.abs().values.flatten(), percentile_cutoff
    )
    subset = data.regulator_coefficients[(data.regulator_coefficients > cut_off).any(axis=1)]
    ax = sns.heatmap(subset, **heatmap_kwargs)
    if savefig_prefix:
        plt.savefig('%s.pdf' % savefig_prefix)
    return ax


def visualize_annotation_associations(
        data: ProteomicsData,
        percentile_cutoff: float = 0,
        savefig_prefix: Optional[str] = None,
        **heatmap_kwargs
):
    """Visualizes the associations between sample annotations and modules.

    Args:
        data: ProteomicsData object with annotation_association_FDR assigned.
        percentile_cutoff: Heatmaps are filtered to show high associations only. What
        percentile threshold should be used.
        savefig_prefix: If prefix is provided, a figure will be saved with this prefix.
        **heatmap_kwargs: Additional keyword args for sns.heatmap

    Returns: matplotlib ax with heatmap of associations

    """
    if data.annotation_association_FDR is None:
        raise KeyError(
            'Must calculate regulator coefficients using '
            'ProteomicsData.calculate_regulator_coefficients before visualizing. '
        )
    temp = -np.log10(data.annotation_association_FDR)
    cut_off = np.nanpercentile(temp.values.flatten(), percentile_cutoff)
    subset = temp[(temp > cut_off).any(axis=1)]
    ax = sns.heatmap(subset, **heatmap_kwargs)
    if savefig_prefix:
        plt.savefig('%s.pdf' % savefig_prefix)
    return ax


def visualize_aa(seq_dfs, save_prefix: Optional[str] = None, **logo_kws):
    """Draws logos of each amino acid sequence motif for each module. Can pass in either
    module_aa_freqs or module_aa_enrichment from ProteomicsData objects.

    Args:
        seq_dfs: Either module_aa_freqs or module_aa_enrichment from ProteomicsData objects.
        save_prefix: If prefix is provided, a figure will be saved with this prefix.
        **logo_kws: Additional keyword args for logomaker.Logo

    Returns: None

    """
    for module, ps in seq_dfs.items():
        logo_kws['color_scheme'] = logo_kws.get('color_scheme', 'NajafabadiEtAl2017')
        logo = logomaker.Logo(ps, **logo_kws)
        logo.ax.set_title('Module %s motif enrichment' % module)
        if save_prefix:
            plt.savefig('%s.logo.motif_enrichment.module%s.pdf' % module)


def visualize_set_enrichment(
        module_enrichment_dict,
        pval_cutoff: float = 0.05,
        save_prefix: Optional[str] = None,
        **barplot_kws
):
    """Draws barplots for either enrichr or ptm-ssGSEA set enrichments per module.

    Args:
        module_enrichment_dict: go_enrichment or ptm_enrichment from ProteomicsData objects.
        pval_cutoff: p-val cut off to filter for significant enrichments.
        save_prefix: If prefix is provided, a figure will be saved with this prefix.
        **barplot_kws: Additional keyword args for sns.barplot

    Returns: None

    """
    barplot_kws['color'] = barplot_kws.get('color', '#BDBDBD')
    for module, df in module_enrichment_dict.items():
        temp = df[df['Adjusted P-value'] < pval_cutoff]
        temp = temp.sort_values('Adjusted P-value')
        if len(temp) > 0:
            fig, axs = plt.subplots(figsize=(6, 0.3 * len(temp)))
            sns.barplot(
                x=-np.log10(temp['Adjusted P-value']),
                y=temp.index,
                ax=axs,
                **barplot_kws
            )
            for i, row in enumerate(temp.iterrows()):
                row = row[1]
                plt.text(0, i, row['Genes'], va='center', fontsize=10)
            plt.xlabel('log$_{10}$(adjusted p-value)')
            plt.ylabel('Terms')
            plt.title('Cluster %s GO terms, FDR %s' % (module, pval_cutoff))
            if save_prefix:
                plt.savefig('%s.barplot.enrichment.pdf')

def visualize_aa_overlap(
        module_overlap_df_dict: dict,
        save_path: Optional[str]=None
        ):
    """
    Plots aa_overlap heatmap for each module.
    Args:
        module_overlap_df_dict: dictionary, output of motif_analysis.aa_overlap_from_df
                                which can be called via ProteomicsData.analyze_aa_overlap
                                keys are modules, values are DataFrames of 
                                aa_overlap scores for each pair of phosphosites
                                within the module
        save_fig:               path to folder where pdfs of plots should be saved.
                                saves no plots if None.
    Returns:
        None
    """
    for module, module_df in module_overlap_df_dict.items():  
        fig_len = 0.5*module_df.shape[0]
        fig_width = 0.4*module_df.shape[1]

        fig = plt.figure(figsize = (fig_len, fig_width))
        sns.heatmap(module_df, xticklabels = module_df.columns, yticklabels = module_df.index)
        plt.title(f'Module {module}')
        if save_path is not None:
            plt.savefig(Path(save_path) / Path(f'heatmap.aa_overlap.module{module}.pdf'))
    plt.show()
    plt.close()



import pandas as pd
from pandas import Series, DataFrame
from gseapy import enrichr
import scipy.stats
from .parsers import read_gmt
from .datasets import load_data
from .utils import multiple_tests_na
from typing import Union


def enrichr_per_module(
        modules: Series,
        background_gene_list,
        gene_sets: str = 'GO_Biological_Process_2018',
        **enrichr_kws
):
    """Runs gseapy.enrichr on genes in each module.

    Args:
        modules: Module Series with sites as the index.
        background_gene_list: List of all unique genes that could have ended up in modules.
        gene_sets: Which gene sets to use. See for options http://amp.pharm.mssm.edu/Enrichr/#stats
        **enrichr_kws: Additional keyword args for gseapy.enrichr()

    Returns: Dictionary of DataFrames with module names as keys and enrichr results as values.

    """
    results = {}
    for module, genes in modules.groupby(modules).groups.items():
        genes = list(set([i[0] for i in genes]))
        res = enrichr(
            gene_list=genes,
            gene_sets=gene_sets,
            background=background_gene_list,
            **enrichr_kws
        )
        res = res.results[[
            'Gene_set',
            'Genes',
            'Overlap',
            'Odds Ratio',
            'P-value',
            'Adjusted P-value',
            'Term'
        ]].set_index('Term')
        results[module] = res
    return results


def ptm_per_module(
        module_seq_dict: dict,
        background_seqs: list,
        ptm_set_gmt: Union[str, dict] = 'human'
):
    """Calculates enrichment for each PTM-ssGSEA set per module via hypergeometric test.

    Args:
        module_seq_dict: Dictionary of amino acid sequences, module names as keys, lists of
        peptide seqs as values.
        background_seqs: List of all peptide seqs that could have ended up in modules, i.e. sites
        that went into the clustering algorithm.
        ptm_set_gmt: str ath to gmt file with PTM-ssGSEA sets to compare against.

    Returns: Dictionary with keys are module names, values are DataFrames with set enrichment
    results per module.

    """
    ptm_set_refs = {
            'human':'ptm.sig.db.all.flanking.human.v1.9.0.gmt',
            'rat':'ptm.sig.db.all.flanking.rat.v1.9.0.gmt',
            'mouse':'ptm.sig.db.all.flanking.mouse.v1.9.0.gmt',
            }
    if type(ptm_set_gmt) == str:
        if ptm_set_gmt in ptm_set_refs.keys():
            ptm_set_gmt = ptm_set_refs[ptm_set_gmt]
            ptm_set_gmt = load_data(ptm_set_gmt, parser=read_gmt)
        else:
            ptm_set_gmt = read_gmt(ptm_set_gmt)
    ptm_set_gmt = {
        k: {item for item in v.items() if item[0] in background_seqs}
        for k, v in ptm_set_gmt.items()
    }
    ptm_set_gmt = {k: v for k, v in ptm_set_gmt.items() if len(v) >= 2}
    
    if len(list(module_seq_dict.values())[0][0]) < 15:
        raise ValueError('Module sequences must be at least 15 AAs long')
    if len(list(module_seq_dict.values())[0][0]) > 15:
        module_seq_dict = {
            k: [seq[int((len(seq)/2-0.5)-7): int((len(seq)/2-0.5)+8)]
                for seq in v] for k, v in module_seq_dict.items()
        }
    background_seqs = set(background_seqs)
    M = len(background_seqs)

    results = {}
    for module, seqs in module_seq_dict.items():
        seqs = set(seqs)
        N = len(seqs)
        module_results = pd.DataFrame(
            columns=[
                'Site_set',
                'Sites',
                'Overlap',
                'P-value',
                'Term'
            ]
        )
        for term, sites in ptm_set_gmt.items():
            sites = dict(sites)
            n = len(sites)
            overlap = seqs.intersection(set(sites.keys()))
            x = len(overlap)
            overlap = [sites[seq] for seq in overlap]
            pval = scipy.stats.hypergeom.sf(x, M, n, N, loc=1)
            line = pd.Series({
                'Site_set':','.join(sites.values()),
                'Sites': ','.join(overlap),
                'Overlap': x,
                'P-value': pval,
                'Term':term
                },
                name=term
            )
            module_results = module_results.append(line)
        module_results['Adjusted P-value'] = multiple_tests_na(
            module_results['P-value'], method='fdr_bh'
        )
        results[module] = module_results.set_index('Term')

    return results



import setuptools

with open("README_pypi.md", "r") as fh:
    long_description = fh.read()
files = ["data/*"]
setuptools.setup(
    name="phosphodisco",
    version="0.0.1",
    author="Tobias Schraink, Ruggles Lab",
    author_email="tschraink@gmail.com",
    description="A package for phosphorylation module discovery and analysis",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ruggleslab/phosphodisco",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Unix",
    ],
    install_requires=[
        "pandas >= 0.24.2",
        "numpy >= 1.16.4",
        "scipy >= 1.2.1",
        "matplotlib >= 3.1.0",
        "seaborn >= 0.9.0",
        "scikit-learn >= 0.22.0",
        "statsmodels >= 0.11.0",
        "hypercluster >= 0.1.13"
    ],
    package_data={"phosphodisco": files},
    entry_points = {
        'console_scripts': [
            'phdc_run=phosphodisco.cli:run',
            'phdc_generate_config=phosphodisco.cli:generate_config'
            ]
        },
    packages=setuptools.find_packages()
)



import phosphodisco as phdc
import numpy as np
import pandas as pd


seed = 5
np.random.seed(seed)
prot = pd.util.testing.makeDataFrame()
phospho = pd.util.testing.makeDataFrame()
phospho.index = pd.MultiIndex.from_tuples(
    [(prot.index[np.random.randint(0, 15)], ind) for ind in phospho.index]
)


def test_classes_regulators():
    proteomics = phdc.ProteomicsData(
        phospho, prot, min_common_values=2
    ).normalize_phospho_by_protein()
    proteomics.impute_missing_values()
    # proteomics.assign_modules()
    proteomics.assign_modules(
        pd.DataFrame(
            {'test;param-1': [np.random.randint(0, 4) for i in range(30)]},
            index=proteomics.normed_phospho.index
        )
    )

    proteomics.calculate_module_scores()
    regs = list(set(phospho.sample(3).index.get_level_values(0)))
    proteomics.collect_possible_regulators(regs, corr_threshold=0.98)
    proteomics.calculate_regulator_association(model='linear', cv_fold=2)
    return proteomics


def test_classes_annotations():
    proteomics = phdc.ProteomicsData(
        phospho, prot, min_common_values=2
    ).normalize_phospho_by_protein()
    proteomics.assign_modules(
        pd.DataFrame(
            {'test;param-1': [np.random.randint(0, 4) for i in range(30)]},
            index=proteomics.normed_phospho.index
        )
    )
    annotations = pd.DataFrame(
            {
                'cat1': ['A', 'B', 'A', 'B'],
                'cat2': ['A', 'B', 'B', 'C'],
                'cont1': [0.115, 0.01, 0.3, 0.9],
                'cont2': [-1, -2.5, np.nan, 1]
            },
            index=proteomics.protein.columns
    )
    proteomics.calculate_module_scores()
    proteomics.add_annotations(annotations, pd.Series(['categorical', 0, 'continuous', 1]))
    proteomics.calculate_annotation_association()
    return proteomics



# phospho = phdc.read_phospho('/Users/lili/dropbox_lili/phosphodisco/results/brca-combined-v4.0-phosphoproteome'
#                    '-dedup-filtered.csv')
# protein = phdc.read_protein(
#     '/Users/lili/dropbox_lili/phosphodisco/results/brca-combined-v4.0-proteome-dedup-filtered.csv')
# normed = phdc.read_phospho(
#     '/Users/lili/dropbox_lili/phosphodisco/results/brca.normed_phospho.csv')
# clusters = phdc.read_phospho('~/dropbox_lili/phosphodisco/results/brca_labels.csv')
# regs = phdc.parsers.read_list(
#     '/Users/lili/dropbox_lili/phosphodisco/phosphodisco/data/kinases_and_phosphatases.txt')
#
# data = phdc.ProteomicsData(
#     phospho=phospho,
#     protein=protein,
#     normed_phospho=normed,
#     modules=clusters,
#     possible_regulator_list=regs
# )
# data.add_annotations(
#     phdc.parsers.read_annotation(
#     '/Users/lili/dropbox_lili/phosphodisco/results/brca-combined-v4.0-sample-annotation.filtered.csv'),
#     pd.Series(phdc.parsers.read_list('/Users/lili/dropbox_lili/phosphodisco/results/brca.annotation_cols.txt'
#                                )),
# )
#
# # data.collect_possible_regulators(corr_threshold=0.9)
# data.calculate_module_scores()
# # data.calculate_regulator_coefficients()
#
# data.calculate_annotation_association(cat_method='RRA')
# data.annotation_association
